library ieee;
use ieee.std_logic_1164.all;
use ieee.std_logic_arith.all;
use ieee.std_logic_unsigned.all;



entity vga_sync is
        port
        (

				-- host side signals (IN)
				
					coe_iCLK_export	: in std_logic;
		  
				-- Device side signals (OUT)
					coe_BLANK_export : out std_logic;
					coe_SYNC_export	: out std_logic;
					coe_HSYNC_export: out std_logic;
					coe_VSYNC_export : out std_logic;
					
					coe_VGAR_export : out std_logic_vector( 9 downto 0 );
					coe_VGAG_export: out std_logic_vector( 9 downto 0);
					coe_VGAB_export	: out std_logic_vector( 9 downto 0 );

					coe_iRed_export : in std_logic_vector(9 downto 0); 
   					coe_iGreen_export : in std_logic_vector(9 downto 0); 
  					coe_iBlue_export : in std_logic_vector(9 downto 0); 
					
					coe_iRST_export : in std_logic;
		          		
					avs_inter_writedata : out std_logic_vector (31 downto 0); 

					clk : in std_logic;

					coe_px_export : out std_logic_vector (9 downto 0); 
					coe_py_export : out std_logic_vector (9 downto 0);  

					coe_video_export : out std_logic := '0';
					
					coe_colRST_export : out std_logic := '0'
                );
end vga_sync;

architecture waffles of vga_sync is 

	-- CONSTANTS!!!
	-- values obtained from http://tinyvga.com/vga-timing/640x480@60Hz

	constant HSYNC_TOTAL_PIX : integer := 800;
	constant HSYNC_SYNC_PULSE : integer := 96;
	constant HSYNC_BACK_PORCH : integer := 48;
	constant HSYNC_FRONT_PORCH : integer := 16;
	constant HSYNC_PIXEL_ACTIVE: integer := 640;
	
	constant VSYNC_TOTAL_PIX : integer := 525;
	constant	VSYNC_SYNC_PULSE : integer := 2;
	constant VSYNC_BACK_PORCH	: integer := 33;
	constant VSYNC_FRONT_PORCH	: integer := 10;
	constant VSYNC_PIXEL_ACTIVE : integer := 480;
	
	
	
	signal hsync :std_logic;
	signal vsync :std_logic;
	signal led_hcount : std_logic;
	signal reset_n: std_logic;
	signal hcount :unsigned(9 downto 0);
	signal vcount : unsigned(9 downto 0);
	
	signal red : unsigned( 9 downto 0 );
	signal green : unsigned( 9 downto 0 );
	signal blue : unsigned( 9 downto 0 );

	signal tempVout: std_logic := '0'; 
	
	signal tempcolRST : std_logic := '0';
	
	signal hPosition : unsigned (9 downto 0) := "0000000000"; 
begin
	
   coe_px_export<= std_logic_vector(hPosition);
	coe_py_export<= std_logic_vector(vcount);

	coe_video_export <= tempVout;
	coe_colRST_export <= tempcolRST;

	--pass along the Pixel clock to the VGA DAC. (Because we're such nice people...)
	
	--tie SYNC_N low. 
	coe_SYNC_export <= '0';
	
	-- blank is high when both hsync and vsync are on.
	coe_HSYNC_export <= hsync;
	coe_VSYNC_export <= vsync;
	coe_BLANK_export <= (hsync and vsync);
	
	--cycle a signal when we finish reading the last line
	rowCount:process(coe_iCLK_export,coe_iRST_export)
	begin
		if rising_edge( coe_iCLK_export) then	
		
		if(hcount = HSYNC_SYNC_PULSE) then
			if (vcount < VSYNC_TOTAL_PIX - VSYNC_FRONT_PORCH) and (vcount > VSYNC_SYNC_PULSE + VSYNC_BACK_PORCH -1) then
				tempVout <= not tempVout;
			end if; 
		end if;
		
		end if; 
	end process rowCount;
	
	colRst:process(clk)
	begin
	
		if(hcount >= HSYNC_SYNC_PULSE + 1 and hcount <= HSYNC_SYNC_PULSE + 4) then
			if (vcount < VSYNC_TOTAL_PIX - VSYNC_FRONT_PORCH + 1) and (vcount > VSYNC_SYNC_PULSE + VSYNC_BACK_PORCH -1) then
				tempcolRST <= '1';
			else
				tempcolRST <= '0';
			end if; 
		else
			tempcolRST <= '0';
		end if;
		
	
	end process colRST;
	

	--process for H.Sync Generation

	process( coe_iCLK_export )
	begin
		if rising_edge( coe_iCLK_export) then
			
			
			if(coe_iRST_export = '1') then
				hcount <= "0000000000";
			elsif hcount < HSYNC_TOTAL_PIX then
				hcount <= hcount + 1;
				
				if( hcount > HSYNC_SYNC_PULSE + HSYNC_BACK_PORCH and hcount < HSYNC_TOTAL_PIX - HSYNC_FRONT_PORCH) then
					hPosition <= hPosition + 1;
				else
					hPosition <= "0000000000";
				end if;
			else
				hcount <= "0000000000";
			end if;
			
			--figure out if we need to cycle the actual hsync signal
			if hcount < HSYNC_SYNC_PULSE then
				hsync <= '0';
			else
				hsync <= '1';
			end if;
			
			
			
		end if;
	end process;
	
	-- Generate vsync.
	process( coe_iCLK_export)
	begin
		if rising_edge( coe_iCLK_export ) then

			-- we need to figure out when the hsync restarts.
			if(coe_iRST_export = '1') then
				vcount <= "0000000000";
			elsif hcount = 0 then
				--vsync counter
				if vcount < VSYNC_TOTAL_PIX then
					vcount <= vcount + 1;
				else
					vcount <= "0000000000";
				end if;
				
				if vcount < VSYNC_SYNC_PULSE then
					vsync <= '0';
				else
					vsync <= '1';
				end if;
				
			end if;
		end if;
	end process;
	
	
	process(coe_iCLK_export)
	begin
		-- pixels can only be output during the active portions.
		if ( (hcount > (HSYNC_SYNC_PULSE + HSYNC_BACK_PORCH)) and (hcount < (HSYNC_TOTAL_PIX - HSYNC_FRONT_PORCH))) 
				and ( (vcount > (VSYNC_SYNC_PULSE + VSYNC_BACK_PORCH -1)) and (vcount < (VSYNC_TOTAL_PIX - VSYNC_FRONT_PORCH)) ) then
			coe_VGAR_export <= coe_iRed_export; 
			coe_VGAG_export <= coe_iGreen_export;
			coe_VGAB_export <= coe_iBlue_export;
		else
			-- not in active space. drive the color lines low. 
			coe_VGAR_export <= 	"0000000000";
			coe_VGAG_export <= 	"0000000000";
			coe_VGAB_export <= 	"0000000000";
		end if;
	end process;
	
	

end architecture waffles;
